﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.IO;
using System.Globalization;
using System.Threading;
using Guardant;
using tlw.Windows.Forms;

namespace clr_form
{
public partial class Form_GrdCode : Form
{
		private Handle hGrd;
		private FileStream fs;
		private FileStream fs_temp;
		private string stdfilename = "C:\\buffer";
		private string curfilename;
		private string tmpfilename = "C:\\tmp";
		private uint dwBufSize = 0;
		private int nDongles = 0;

		private uint dwPublic = 0x519175b7;
		private uint dwPrivRD = 0x51917645;
		private uint dwPrivWR = 0x51917603;
		private uint dwPrivMS = 0x5191758c;

		private string szAsciiExt = "< custom >";
		private uint dwPublicExt = 0;
		private uint dwPrivRDExt = 0;
		private uint dwPrivWRExt = 0;
		private uint dwPrivMSExt = 0;

		private uint[] dwAccessCode = new uint[50];

		private byte[] byNsdMask = new byte[0x10000];
		private byte[] byExtractMask = new byte[4096];
		private byte[] byLoadableCode = new byte[0];

		private byte[] byTRUkey = new byte[16];

		DynamicFileByteProvider dynamicFileByteProvider;

		public Form_GrdCode()
		{
			InitializeComponent();

			curfilename = stdfilename;

			dwBufSize = Math.Max(Convert.ToUInt32(textBox_sizein.Text, 16), Convert.ToUInt32(textBox_sizeout.Text, 16));
			ReCreateBufferFile(curfilename, dwBufSize, false);
			PrintBuffer();

			if (dynamicFileByteProvider == null)
			{
				stdfilename = "buffer";
				curfilename = stdfilename;
				ReCreateBufferFile(curfilename, dwBufSize, false);
				PrintBuffer();
			}
		}

		private void SetStatus(string message, Color color)
		{
			StatusLabel_text.Text = message;
			StatusLabel_text.ForeColor = color;
		}

		private void OperationFailed(string sFunctionName)
		{
			if (sFunctionName != "GrdStartup")
			{
				throw new OperationFailedException(sFunctionName);
			}
		}

		private void OperationFailedExceptionHandler(string sFunctionName)
		{
			string str1;
			string str2;
			GrdApi.GrdFormatMessage(GrdApi.GrdGetLastError(hGrd), GrdLNG.English, out str2);
			str1 = String.Format("Error occured! " + sFunctionName + " failed, errcode: {0:D}", GrdApi.GrdGetLastError(hGrd));
			SetStatus(str1 + " (" + str2 + ")", Color.Red);
		}

		private bool isAccessCodesKnown(uint dwPublicFound)
		{
			if (dwPublicFound == 0x519175b7)
			{
				dwPublic = 0x519175b7;
				dwPrivRD = 0x51917645;
				dwPrivWR = 0x51917603;
				dwPrivMS = 0x5191758c;
			}
			else if (dwPublicFound == 0xB33F7308)
			{
				dwPublic = 0xB33F7308;
				dwPrivRD = 0xD21C2838;
				dwPrivWR = 0xD2EDAAE4;
				dwPrivMS = 0x0E1F5953;
			}
			else if (dwPublicFound == dwPublicExt)
			{
				dwPublic = dwPublicExt;
				dwPrivRD = dwPrivRDExt;
				dwPrivWR = dwPrivWRExt;
				dwPrivMS = dwPrivMSExt;
			}
			else
				return false;

			return true;
		}

		private void AddDongle(uint id, uint dwPublic)
		{
			string str = String.Format("{0:X}", id);
			listBox_ID.Items.Add(str);
			dwAccessCode[nDongles] = dwPublic;
			nDongles++;
		}

		private void DongleFound(uint ID, int dwPublic)
		{
			if (isAccessCodesKnown((uint)dwPublic))
				AddDongle(ID, (uint)dwPublic);
		}

		private void ReleaseFile()
		{
			if (dynamicFileByteProvider != null)
			{
				dynamicFileByteProvider.ApplyChanges();
				dynamicFileByteProvider.Dispose();
				dynamicFileByteProvider = null;
			}

			if (fs != null)
				fs.Close();
		}

		private void PrintBuffer()
		{
			ReleaseFile();

			try
			{
				dynamicFileByteProvider = new DynamicFileByteProvider(curfilename);
				dynamicFileByteProvider.LengthChanged += new System.EventHandler(this.dynamicFileByteProvider_LengthChanged);
				hexBox.ByteProvider = dynamicFileByteProvider;
			}
			catch (IOException)
			{
				try
				{
					SetStatus("I/O exception occured on File Read operation! Default buffer is used!", Color.Blue);
					curfilename = stdfilename;
					dynamicFileByteProvider = new DynamicFileByteProvider(curfilename);
					dynamicFileByteProvider.LengthChanged += new System.EventHandler(this.dynamicFileByteProvider_LengthChanged);
					hexBox.ByteProvider = dynamicFileByteProvider;
				}
				catch (IOException)
				{
					SetStatus("I/O exception occured on File Read operation!", Color.Red);
				}
			}

			return;
		}

		private long ReadHexBuf(byte[] buf)
		{
			long i;

			for (i = 0; i < dynamicFileByteProvider.Length && i < buf.Length; ++i)
			{
				if (i < buf.Length)
					buf[i] = dynamicFileByteProvider.ReadByte(i);
			}

			return i;
		}

		private void WriteHexBuf(byte[] buf)
		{
			for (long i = 0; i < dynamicFileByteProvider.Length; ++i)
			{
				if (i < buf.Length)
					dynamicFileByteProvider.WriteByte(i, buf[i]);
				else
					dynamicFileByteProvider.WriteByte(i, 0);
			}

			hexBox.Refresh();
		}

		private string Numeric(string str_in)
		{
			char[] a = str_in.ToCharArray();
			char[] b = new char[8];
			int nCounter = 0;

			foreach(char ch in a)
			{
				if ("1234567890abcdefABCDEF".IndexOf(ch) != -1)
				{
					b[nCounter] = ch;
					nCounter++;
				}
			}

			return new string(b, 0, Array.IndexOf(b, '\0')); ;
		}

		private uint GetDescriptorOffset(uint numericName)
		{
			uint i;

			for (i = 0x20; i < byExtractMask[0x12] + 0x100 * byExtractMask[0x13] - 0x1e; i += 2)
				if (byExtractMask[i] + 0x100 * byExtractMask[i + 1] == numericName)
					break;

			if (i >= byExtractMask[0x12] + 0x100 * byExtractMask[0x13] - 0x1e)
				return 0;

			i += (uint)byNsdMask[0x2c] * 2;
			return (uint)(byExtractMask[i] + 0x100 * byExtractMask[i + 1]);
		}

		private bool isCodeDescriptor(uint numericName)
		{
			uint offset = GetDescriptorOffset(numericName);

			if (offset == 0)
				SetStatus("Error occured! No such descriptor in mask!", Color.Red);
			else
			{
				if (byExtractMask[offset - 0x1e + 1] == 0x0e)
					return true;
				else
					return false;
			}

			return false;
		}

		private uint SearchNextCodeDescriptor()
		{
			uint i = 0x20;
			uint current = 0;

			if (textBox_nn_code.Text.Length == 0)
				return 0x10000;

			current = Convert.ToUInt32(textBox_nn_code.Text, 16);

			for (i = 0x20; i < byExtractMask[0x12] + 0x100 * byExtractMask[0x13] - 0x1e; i += 2)
				if (byExtractMask[i] + 0x100 * byExtractMask[i + 1] == current)
					break;

			i += 2;

			while (true)
			{
				if (byExtractMask[i] + 0x100 * byExtractMask[i + 1] == current)
					break;

				if (i >= 0x20 + (uint)byNsdMask[0x2c] * 2)
					i = 0x1e;
				else
				{
					if (isCodeDescriptor((uint)(byExtractMask[i] + 0x100 * byExtractMask[i + 1])))
						break;
				}

				i += 2;
			}

			// return numeric name of Code, 0x10000 otherwise

			if (byExtractMask[i] + 0x100 * byExtractMask[i + 1] == current && !isCodeDescriptor(current))
				return 0x10000;
			else
				return (uint)(byExtractMask[i] + 0x100 * byExtractMask[i + 1]);
		}

		private void SetCodeParameters(uint numericName)
		{
			string str1 = "";
			string str2 = "";
			uint i = GetDescriptorOffset(numericName) - 0x1e;

			str1 = String.Format("{0:X}",
					byExtractMask[i + 0x90] +
					byExtractMask[i + 0x91] * 0x100 +
					byExtractMask[i + 0x92] * 0x10000 +
					byExtractMask[i + 0x93] * 0x1000000);
			str2 = String.Format("{0:X}",
					byExtractMask[i + 0x94] +
					byExtractMask[i + 0x95] * 0x100 +
					byExtractMask[i + 0x96] * 0x10000 +
					byExtractMask[i + 0x97] * 0x1000000);

			if (byExtractMask[i + 0x97] < 0x10)
				label_ROM_prarms.Text = "0".PadRight((8 - str1.Length), '0') + str1 + ":" + "0".PadRight((8 - str2.Length), '0') + str2;
			else
				label_ROM_prarms.Text = str1 + ":" + str2;

			str1 = String.Format("{0:X}",
					byExtractMask[i + 0x98] +
					byExtractMask[i + 0x99] * 0x100 +
					byExtractMask[i + 0x9a] * 0x10000 +
					byExtractMask[i + 0x9b] * 0x1000000);
			str2 = String.Format("{0:X}",
					byExtractMask[i + 0x9c] +
					byExtractMask[i + 0x9d] * 0x100 +
					byExtractMask[i + 0x9e] * 0x10000 +
					byExtractMask[i + 0x9f] * 0x1000000);

			if (byExtractMask[i + 0x9f] < 0x10)
				label_RAM_prarms.Text = "0".PadRight((8 - str1.Length), '0') + str1 + ":" + "0".PadRight((8 - str2.Length), '0') + str2;
			else
				label_RAM_prarms.Text = str1 + ":" + str2;
		}

		private void ReCreateBufferFile(string filename, uint length, bool isReinitializing)
		{
			try
			{
				if (isReinitializing)
				{
					ReleaseFile();

					if (File.Exists(tmpfilename))
						File.Delete(tmpfilename);

					if (File.Exists(curfilename))
						File.Delete(curfilename);

					fs = File.Create(curfilename);

					for (uint i = 0; i < length; ++i)
						fs.WriteByte(0);
				}
				else
				{
					if (File.Exists(curfilename))
					{
						ReleaseFile();

						if (File.Exists(tmpfilename))
							File.Delete(tmpfilename);

						File.Move(curfilename, tmpfilename);
						fs_temp = File.Open(tmpfilename, FileMode.Open);
						fs = File.Create(curfilename);

						if (fs == null || fs_temp == null)
						{
							SetStatus("Error occured! Operation with buffer file failed!", Color.Red);

							if (fs_temp != null) fs_temp.Close();

							if (fs != null) fs.Close();

							return;
						}

						for (uint i = 0; i < length; ++i)
						{
							if (i < fs_temp.Length)
								fs.WriteByte((byte)fs_temp.ReadByte());
							else
								fs.WriteByte(0);
						}

						fs_temp.Close();
						File.Delete(tmpfilename);
						fs.Close();
					}
					else
					{
						ReleaseFile();
						fs = File.Create(curfilename);

						for (uint i = 0; i < length; ++i)
							fs.WriteByte(0);

						fs.Close();
					}
				}
			}
			catch (IOException)
			{

			}
		}

		private void button_clean_Click(object sender, EventArgs e)
		{
			if (textBox_sizein.Text.Length == 0 || textBox_sizeout.Text.Length == 0)
				return;

			dwBufSize = Math.Max(Convert.ToUInt32(Numeric(textBox_sizein.Text), 16), Convert.ToUInt32(Numeric(textBox_sizeout.Text), 16));
			ReCreateBufferFile(curfilename, dwBufSize, true);
			PrintBuffer();
		}

		private void button_opennsd_Click(object sender, EventArgs e)
		{
			openFileDialog_nsd.CheckFileExists = true;
			openFileDialog_nsd.Filter = "Guardant Mask (*.nsd)|*.nsd|All files (*.*)|*.*";
			openFileDialog_nsd.ShowDialog();
		}

		private void openFileDialog_nsd_FileOk(object sender, CancelEventArgs e)
		{
			try
			{
				XmlTextReader reader = new XmlTextReader(openFileDialog_nsd.FileName);
				reader.WhitespaceHandling = WhitespaceHandling.None;

				while (reader.Read() && reader.Name != "Data") ;

				reader.Read();
				string s = reader.Value;
				string[] ss = s.Split(' ');

				for (int i = 0; i < System.Math.Min(0x10000, ss.Length - 1); ++i) byNsdMask[i] = (byte)Convert.ToUInt16(ss[i], 16);
			}
			catch (IOException)
			{
				SetStatus("I/O exception occured on File Read operation!", Color.Red);
			}

			textBox_nsd.Text = openFileDialog_nsd.FileName;
			Array.Copy(byNsdMask, 0x196, byExtractMask, 0, 4096);

			if (byExtractMask[0x0e] != 0 || byExtractMask[0x0f] != 1)
			{
				SetStatus("Error occured! Type of Mask selected does not supported!", Color.Red);
				return;
			}

			if (isCodeDescriptor(Convert.ToUInt32(textBox_nn_code.Text, 16)))
				SetCodeParameters(Convert.ToUInt32(textBox_nn_code.Text, 16));
			else
			{
				uint NN = SearchNextCodeDescriptor();

				if (NN == 0x10000)
				{
					SetStatus("Error occured! Code descriptor have not found in mask!", Color.Red);
					return;
				}
				else
					textBox_nn_code.Text = "0".PadRight((4 - String.Format("{0:X}", NN).Length), '0') + String.Format("{0:X}", NN);
			}

			SetStatus("Mask has been successfully uploaded.", Color.Green);
		}

		private void button_opengcexe_Click(object sender, EventArgs e)
		{
			openFileDialog_gcexe.CheckFileExists = true;
			openFileDialog_gcexe.Filter = "Loadable Code (*.gcexe)|*.gcexe|All files (*.*)|*.*";
			openFileDialog_gcexe.ShowDialog();
		}

		private void openFileDialog_gcexe_FileOk(object sender, CancelEventArgs e)
		{
			try
			{
				FileStream str = new FileStream(openFileDialog_gcexe.FileName, FileMode.Open);
				BinaryReader br = new BinaryReader(str);
				byLoadableCode = br.ReadBytes(0x20000);
				br.Close();
				SetStatus("Loadable Code has been successfully uploaded.", Color.Green);
			}
			catch (IOException)
			{
				SetStatus("I/O exception occured on File Read operation!", Color.Red);
			}
			textBox_gcexe.Text = openFileDialog_gcexe.FileName;
		}

		private void ResetDonglesList()
		{
			listBox_ID.Items.Clear();
			nDongles = 0;
		}

		private void button_search_Click(object sender, EventArgs e)
		{
			Splash splash = new Splash();
			splash.ShowDialog();

			uint ID = 0;
			GrdE Ret;
			FindInfo pFindInfo;

			ResetDonglesList();

			try
			{
				if (GrdApi.GrdStartup(GrdFMR.Local) != GrdE.OK)
					OperationFailed("GrdStartup");

				hGrd = GrdApi.GrdCreateHandle(GrdCHM.SingleThread);

				if (hGrd.Address.Equals(0))
					OperationFailed("GrdCreateHandle");

				if (GrdApi.GrdSetFindMode(hGrd, GrdFMR.Local, 0, 0, 0, 0, 0, 0, 0, GrdFMM.CodeUSB, GrdFMI.ALL) != GrdE.OK)
					OperationFailed("GrdSetFindMode");

				Ret = GrdApi.GrdFind(hGrd, GrdF.First, out ID, out pFindInfo);

				while (Ret == GrdE.OK)
				{
					DongleFound(ID, pFindInfo.dwPublicCode);
					Ret = GrdApi.GrdFind(hGrd, GrdF.Next, out ID, out pFindInfo);
				}

				if (listBox_ID.Items.Count > 0)
				{
					listBox_ID.SelectedIndex = 0;
					SetStatus("Dongle search successfully complete!", Color.Green);
				}
				else
					OperationFailed("GrdFind");

			}
			catch (OperationFailedException ex)
			{
				OperationFailedExceptionHandler(ex.szFunctionName);
			}
			finally
			{
				GrdApi.GrdCloseHandle(hGrd);
				GrdApi.GrdCleanup();
			}

			splash.Close();
		}

		private void button_grdcodeload_Click(object sender, EventArgs e)
		{
			Splash splash = new Splash();
			splash.ShowDialog();

			uint ID = 0;

			if (byLoadableCode.Length == 0)
			{
				SetStatus("GrdCodeLoad failed. Please, set loadablecode path at first.", Color.Red);
			}
			else
			{
				if (listBox_ID.GetItemText(listBox_ID.SelectedItem).Length == 0)
				{
					SetStatus("GrdCodeLoad failed. Please, search for dongle at first.", Color.Red);
				}
				else
				{
					ID = Convert.ToUInt32(listBox_ID.GetItemText(listBox_ID.SelectedItem), 16);

					try
					{
						if (GrdApi.GrdStartup(GrdFMR.Local) != GrdE.OK)
							OperationFailed("GrdStartup");

						hGrd = GrdApi.GrdCreateHandle(GrdCHM.SingleThread);

						if (hGrd.Address.Equals(0))
							OperationFailed("GrdCreateHandle");

						if (GrdApi.GrdSetFindMode(hGrd, GrdFMR.Local, GrdFM.ID, 0, ID, 0, 0, 0, 0, GrdFMM.CodeUSB, GrdFMI.ALL) != GrdE.OK)
							OperationFailed("GrdSetFindMode");

						if (GrdApi.GrdSetAccessCodes(hGrd, dwPublic, dwPrivRD, dwPrivWR, dwPrivMS) != GrdE.OK)
							OperationFailed("GrdSetAccessCodes");

						if (GrdApi.GrdLogin(hGrd, GrdLM.PerHandle) != GrdE.OK)
							OperationFailed("GrdLogin");

						if (GrdApi.GrdCodeLoad(hGrd, Convert.ToUInt32(textBox_nn_code.Text, 16), byLoadableCode.Length, byLoadableCode) != GrdE.OK)
							OperationFailed("GrdCodeLoad");

						SetStatus("GrdCodeLoad successfully finished.", Color.Green);
					}
					catch (OperationFailedException ex)
					{
						OperationFailedExceptionHandler(ex.szFunctionName);
					}
					finally
					{
						GrdApi.GrdCloseHandle(hGrd);
						GrdApi.GrdCleanup();
					}
				}
			}

			splash.Close();
		}

		private void button_WriteMask_Click(object sender, EventArgs e)
		{
			//int threadId;

			//AsyncDemo ad = new AsyncDemo();
			//AsyncMethodCaller caller = new AsyncMethodCaller(ad.SplashAsyncMethod);
			//IAsyncResult result = caller.BeginInvoke(5000, out threadId, null, null);
			//Thread.Sleep(0);
			//ad.splash.ShowDialog();

			Splash splash = new Splash();
			splash.ShowDialog();

			uint ID = 0;

			if (byNsdMask.Max() == 0)
			{
				SetStatus("Error writing mask. Please, set mask path at first.", Color.Red);
			}
			else
			{
				try
				{
					if (GrdApi.GrdStartup(GrdFMR.Local) != GrdE.OK)
						OperationFailed("GrdStartup");

					hGrd = GrdApi.GrdCreateHandle(GrdCHM.SingleThread);

					if (hGrd.Address.Equals(0))
						OperationFailed("GrdCreateHandle");

					if (listBox_ID.GetItemText(listBox_ID.SelectedItem).Length == 0)
					{
						SetStatus("Error writing mask. Please, search for dongle at first.", Color.Red);
					}
					else
					{
						ID = Convert.ToUInt32(listBox_ID.GetItemText(listBox_ID.SelectedItem), 16);

						if (GrdApi.GrdSetAccessCodes(hGrd, dwPublic, dwPrivRD, dwPrivWR, dwPrivMS) != GrdE.OK)
							OperationFailed("GrdSetAccessCodes");

						if (GrdApi.GrdLogin(hGrd, GrdLM.PerHandle) != GrdE.OK)
							OperationFailed("GrdLogin");

						for (int i = 0; i < 16; ++i) byTRUkey[i] = 0;

						if (GrdApi.GrdTRU_SetKey(hGrd, byTRUkey) != GrdE.OK)
							OperationFailed("GrdTRU_SetKey");

						if (GrdApi.GrdWrite(hGrd,
								GrdUAM.bNProg,
								byExtractMask) != GrdE.OK)
							OperationFailed("GrdWrite");

						if (GrdApi.GrdProtect(hGrd,
								(uint)(byNsdMask[0x1a] + byNsdMask[0x1b] * 0x100),
								(uint)(byNsdMask[0x1c] + byNsdMask[0x1d] * 0x100),
								byNsdMask[0x2c],
								byNsdMask[0x2d]) != GrdE.OK)
							OperationFailed("GrdProtect");

						SetStatus("Well done! Mask has been successfully written!", Color.Green);
					}
				}
				catch (OperationFailedException ex)
				{
					OperationFailedExceptionHandler(ex.szFunctionName);
				}
				finally
				{
					GrdApi.GrdCloseHandle(hGrd);
					GrdApi.GrdCleanup();
				}
			}

			splash.Close();
			//ad.splash.Hide();
			//string returnValue = caller.EndInvoke(out threadId, result);
		}

		private void button_grdcoderun_Click(object sender, EventArgs e)
		{
			Splash splash = new Splash();
			splash.ShowDialog();

			uint ID = 0;
			uint Ret = 0;
			string str = "";

			byte[] byBufferToDongle = new byte[0x1000000];
			byte[] byBufferFromDongle;

			if (listBox_ID.GetItemText(listBox_ID.SelectedItem).Length == 0)
			{
				SetStatus("GrdCodeRun failed. Please, search for dongle at first.", Color.Red);
			}
			else
			{
				ID = Convert.ToUInt32(listBox_ID.GetItemText(listBox_ID.SelectedItem), 16);

				try
				{
					ReadHexBuf(byBufferToDongle);

					if (GrdApi.GrdStartup(GrdFMR.Local) != GrdE.OK)
						OperationFailed("GrdStartup");

					hGrd = GrdApi.GrdCreateHandle(GrdCHM.SingleThread);

					if (hGrd.Address.Equals(0))
						OperationFailed("GrdCreateHandle");

					if (GrdApi.GrdSetFindMode(hGrd, GrdFMR.Local, GrdFM.ID, 0, ID, 0, 0, 0, 0, GrdFMM.CodeUSB, GrdFMI.ALL) != GrdE.OK)
						OperationFailed("GrdSetFindMode");

					if (GrdApi.GrdSetAccessCodes(hGrd, dwPublic, dwPrivRD, dwPrivWR, dwPrivMS) != GrdE.OK)
						OperationFailed("GrdSetAccessCodes");

					if (GrdApi.GrdLogin(hGrd, GrdLM.PerHandle) != GrdE.OK)
						OperationFailed("GrdLogin");

					if (GrdApi.GrdCodeRun(hGrd,
							Convert.ToUInt32(textBox_nn_code.Text, 16),
							Convert.ToUInt32(textBox_param_retvalue.Text, 16),
							out Ret,
							Convert.ToUInt32(textBox_sizeout.Text, 16),
							out byBufferFromDongle,
							Convert.ToUInt32(textBox_sizein.Text, 16),
							byBufferToDongle) != GrdE.OK)
						OperationFailed("GrdCodeRun");

					WriteHexBuf(byBufferFromDongle);
					str = String.Format("{0:X}", Ret);

					if (Ret < 0x10000000)
						textBox_param_retvalue.Text = "0".PadRight((8 - str.Length), '0') + str;
					else
						textBox_param_retvalue.Text = str;

					SetStatus("GrdCodeRun successfully finished.", Color.Green);
				}
				catch (OperationFailedException ex)
				{
					OperationFailedExceptionHandler(ex.szFunctionName);
				}
				catch (IndexOutOfRangeException)
				{
					SetStatus("IndexOutOfRange exception occured! Please, fix SizeIn and SizeOut values!", Color.Red);
				}
				catch (FormatException)
				{
					SetStatus("IndexOutOfRange exception occured! Please, fix SizeIn and SizeOut values!", Color.Red);
				}
				finally
				{
					GrdApi.GrdCloseHandle(hGrd);
					GrdApi.GrdCleanup();
				}
			}

			splash.Close();
		}

		private void textBox_sizeout_TextChanged(object sender, EventArgs e)
		{
			if (textBox_sizein.Text.Length == 0 || textBox_sizeout.Text.Length == 0)
				return;

			if (Numeric(textBox_sizein.Text) == "" || Numeric(textBox_sizeout.Text) == "")
				return;

			dwBufSize = Math.Max(Convert.ToUInt32(Numeric(textBox_sizein.Text), 16), Convert.ToUInt32(Numeric(textBox_sizeout.Text), 16));
			ReCreateBufferFile(curfilename, dwBufSize, false);
			PrintBuffer();
		}

		private void button_nextcode_Click(object sender, EventArgs e)
		{
			uint NN = SearchNextCodeDescriptor();

			if (NN == 0x10000)
				SetStatus("Error occured! Code descriptor have not found in mask!", Color.Red);
			else
				textBox_nn_code.Text = "0".PadRight((4 - String.Format("{0:X}", NN).Length), '0') + String.Format("{0:X}", NN);
		}

		private void textBox_nn_code_TextChanged(object sender, EventArgs e)
		{
			uint NN = 0;

			if (textBox_nn_code.Text.Length > 0)
			{
				NN = Convert.ToUInt16(textBox_nn_code.Text, 16);
				SetCodeParameters(NN);
			}
		}

		private void dynamicFileByteProvider_LengthChanged(object sender, EventArgs e)
		{
			textBox_sizein.Text = String.Format("{0:X}", dynamicFileByteProvider.Length);
		}

		private void listBox_ID_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (isAccessCodesKnown(dwAccessCode[listBox_ID.SelectedIndex]))
				SetStatus("A dongle selected. Public code is 0x" + String.Format("{0:X}", dwAccessCode[listBox_ID.SelectedIndex]), Color.Green);
			else
				SetStatus("Unknown public code (0x" + String.Format("{0:X}", dwAccessCode[listBox_ID.SelectedIndex]) + ")", Color.Red);
		}

		private void button_accesscodes_Click(object sender, EventArgs e)
		{
			AccessCodes ac = new AccessCodes();
			ac.acsiiExt = szAsciiExt;
			ac.dwPublicExt = dwPublicExt;
			ac.dwPrivRDExt = dwPrivRDExt;
			ac.dwPrivWRExt = dwPrivWRExt;
			ac.dwPrivMSExt = dwPrivMSExt;
			ac.ShowDialog();
			szAsciiExt = ac.acsiiExt;
			dwPublicExt = ac.dwPublicExt;
			dwPrivRDExt = ac.dwPrivRDExt;
			dwPrivWRExt = ac.dwPrivWRExt;
			dwPrivMSExt = ac.dwPrivMSExt;
		}

		private void Form_GrdCode_FormClosing(object sender, FormClosingEventArgs e)
		{
			if (File.Exists(stdfilename))
			{
				ReleaseFile();
				File.Delete(stdfilename);
			}
		}

		private void button_saveas_Click(object sender, EventArgs e)
		{
			saveFileDialog_buffer.Filter = "All files (*.*)|*.*";
			saveFileDialog_buffer.ShowDialog();
		}

		private void saveFileDialog_buffer_FileOk(object sender, CancelEventArgs e)
		{
			byte[] byFileBuffer = new byte[0x20000];
			long length = ReadHexBuf(byFileBuffer);
			FileStream stream = new FileStream(saveFileDialog_buffer.FileName, FileMode.CreateNew);
			BinaryWriter bw = new BinaryWriter(stream);
			bw.Write(byFileBuffer, 0, (int)length);
			bw.Close();
			SetStatus("Buffer has been successfully saved.", Color.Green);
		}

		private void hexBox_DragEnter(object sender, System.Windows.Forms.DragEventArgs e)
		{
			// Enables Drag-n-Drop
			e.Effect = DragDropEffects.All;
		}

		private void cut_Click(object sender, EventArgs e)
		{
			this.hexBox.Cut();
		}

		private void copy_Click(object sender, EventArgs e)
		{
			this.hexBox.Copy();
		}

		private void paste_Click(object sender, EventArgs e)
		{
			this.hexBox.Paste();
		}

		private void copyHex_Click(object sender, EventArgs e)
		{
			this.hexBox.CopyHex();
		}

		private void pasteHex_Click(object sender, EventArgs e)
		{
			this.hexBox.PasteHex();
		}

		private void hexBox_DragDrop(object sender, DragEventArgs e)
		{
			string[] formats = e.Data.GetFormats();
			object oFileNames = e.Data.GetData(DataFormats.FileDrop);
			string[] fileNames = (string[])oFileNames;

			if (fileNames.Length == 1)
			{
				curfilename = fileNames[0];
			}

			try
			{
				fs = File.Open(curfilename, FileMode.Open);

				if (fs.Length < 0x1000000)
				{
					if (fs.Length == 0)
					{
						for (int i = 0; i < 0x20; ++i)
							fs.WriteByte(0);
					}

					textBox_sizein.Text = String.Format("{0:X}", fs.Length);
					textBox_sizeout.Text = String.Format("{0:X}", fs.Length);
					SetStatus("", Color.Black);
				}
				else
					SetStatus("Too large file selected!", Color.Red);

				fs.Close();
			}
			catch (IOException)
			{
				SetStatus("I/O Exception occured! Please, select another file!", Color.Red);
			}
		}
}
}